from __future__ import print_function, absolute_import

import json
import os
import sys
import time

import wx

THISFILE = os.path.abspath(sys.argv[0])
THISPATH = os.path.dirname(THISFILE)
if THISPATH not in sys.path:
    sys.path.append(THISPATH)

if sys.version_info.major > 2:
    # Python 3 (Poser 12 and above)
    basestring = str
    map = lambda a, b: [a(_b) for _b in b]
else:
    # Python 2 (Poser 11 and below)
    if not os.path.exists(os.path.join(THISPATH, "__init__.py")):
        # this file must exist to be able to make includes
        with open(os.path.join(THISPATH, "__init__.py"), "w"):
            pass

try:
    import poser
except ImportError:
    import POSER_FAKE as poser

import im_export

reload(im_export)
try:
    from .im_export import do_import, do_export
    # from .CombineMorphDialog import Dialog as CombineDialog
    from . import WXtools
except ValueError:
    from im_export import do_import, do_export
    # from CombineMorphDialog import Dialog as CombineDialog
    import WXtools

aui = poser.WxAuiManager()

SCENE = poser.Scene()
TITLE = "OBJ Im-/Export"
TEMPDIR = poser.TempLocation()
IMEX_CONFIG = globals().setdefault("IMEX_CONFIG", dict()) or dict()
IMEX_CONFIG["THISPATH"] = os.path.dirname(sys.argv[0])
im_export.IMEX_CONFIG = IMEX_CONFIG
TODAYS_MORPHNAME = "MORPH" + time.strftime("%m%d")
LIGHT = 0xf0, 0xf0, 0xf0
DARK = 0x4b, 0x4b, 0x4b
BTN_COLOR = 0x70, 0x70, 0x70
INPUT_BG_COLOR = 0x60, 0x60, 0x60


def load_config():
    global IMEX_CONFIG
    fname = os.path.join(THISPATH, "imex.config")
    if os.path.exists(fname) and os.access(fname, os.R_OK):
        with open(os.path.join(THISPATH, "imex.config")) as fh:
            IMEX_CONFIG = json.load(fh)
        return True
    return False


def save_config():
    global IMEX_CONFIG
    fname = os.path.join(THISPATH, "imex.config")
    with open(fname, "w") as fh:
        json.dump(IMEX_CONFIG, fh, indent=4)


class Menu(wx.Panel):
    _CONFIGCHANGED = False

    def __init__(self, parent,
                 name=TITLE,
                 style=wx.DEFAULT_FRAME_STYLE,
                 size=(270, 300),
                 **kwargs):

        WXtools.CONFIG = IMEX_CONFIG

        super(self.__class__, self).__init__(parent=parent,
                                             id=wx.ID_ANY,
                                             name=name,
                                             style=style,
                                             size=size,
                                             **kwargs)
        load_config()
        IMEX_CONFIG["BackgroundColor"] = DARK
        IMEX_CONFIG["ForegroundColor"] = LIGHT
        self.SetBackgroundColour(DARK)
        self.SetForegroundColour(LIGHT)

        self.gb_manager = WXtools.GridBagManager(parent=self,
                                                 sizer=wx.GridBagSizer(2, 4),
                                                 flag=wx.ALL | wx.ALIGN_CENTER_VERTICAL | wx.EXPAND,
                                                 border=2
                                                 )

        self.figurelist = WXtools.PrepCtrl(wx.Choice, choices=[],
                                           style=wx.CB_SORT | wx.BORDER_NONE,
                                           name="figurelist",
                                           tooltip="Right click to reload Figurelist.",
                                           bind=((wx.EVT_RIGHT_UP, self.updateFigurelist),
                                                 (wx.EVT_CHOICE, self.onChoice))
                                           )
        self.updateFigurelist()
        self.SetAutoLayout(True)
        self.initUI(self.gb_manager)
        self.Bind(wx.EVT_WINDOW_DESTROY, self.onClose)
        self.Bind(wx.EVT_MOUSE_EVENTS, lambda ev: ev.StopPropagation())

    def initUI(self, manager):
        isinstance(manager, WXtools.GridBagManager)
        labelfont = root.GetFont()  # type: wx.Font
        labelfont.SetWeight(wx.FONTWEIGHT_BOLD)
        manager.addrow(WXtools.LabelCtrl(label="Figure", font=labelfont), self.figurelist)
        manager.addrow(WXtools.LabelCtrl(label="Scale", font=labelfont),
                       WXtools.PrepCtrl(wx.TextCtrl, name="TX_Scale",
                                        bgcolor=INPUT_BG_COLOR,
                                        tooltip="Scale for OBJ export and import.\n"
                                                "Insert a number in the range 1-1000",
                                        style=wx.ALIGN_RIGHT | wx.BORDER_NONE,
                                        value=str(IMEX_CONFIG.get("TX_Scale", 1))),
                       flag=wx.EXPAND)
        manager.addrow(WXtools.LabelCtrl(label="Morph Name", font=labelfont),
                       WXtools.PrepCtrl(wx.TextCtrl, name="TX_Morphname",
                                        bgcolor=INPUT_BG_COLOR,
                                        tooltip="Morphname used in all actors for next imported morph",
                                        style=wx.ALIGN_RIGHT | wx.BORDER_NONE,
                                        value=str(IMEX_CONFIG.get("morphname", TODAYS_MORPHNAME))),
                       flag=wx.EXPAND)

        manager.addrow(None)
        manager.addrow(WXtools.LabelCtrl(label="Export Groups", font=labelfont),
                       WXtools.PrepCtrl(wx.CheckBox, name="FLAG_UseGroups",
                                        tooltip="Write actor Groupnames to export file",
                                        style=wx.ALIGN_RIGHT | wx.BORDER_NONE,
                                        value=IMEX_CONFIG.get("FLAG_UseGroups", False)))
        manager.addrow(WXtools.LabelCtrl(label="Export UV info", font=labelfont),
                       WXtools.PrepCtrl(wx.CheckBox, name="FLAG_UseMat",
                                        tooltip="Write UV-/Texture info to export file",
                                        style=wx.ALIGN_RIGHT | wx.BORDER_NONE,
                                        value=IMEX_CONFIG.get("FLAG_UseMat", True)))
        manager.addrow(WXtools.LabelCtrl(label="Copy Imagefiles", font=labelfont),
                       WXtools.PrepCtrl(wx.CheckBox, name="FLAG_ImgCopy",
                                        tooltip="Copy texture files to export path",
                                        style=wx.ALIGN_RIGHT,
                                        value=IMEX_CONFIG.get("FLAG_ImgCopy", True)))
        manager.addrow(None)
        manager.addrow(WXtools.LabelCtrl(label="Silent Im-/Export"),
                       WXtools.PrepCtrl(wx.CheckBox, name="FLAG_SilentImExport",
                                        tooltip="Im-/Export to/from file without Dialog if possible.",
                                        style=wx.ALIGN_RIGHT,
                                        value=IMEX_CONFIG.get("FLAG_SilentImExport", False)))
        manager.addrow(WXtools.LabelCtrl(label="X Center == 0"),
                       WXtools.PrepCtrl(wx.CheckBox, name="FLAG_ZeroCenter",
                                        tooltip="Forces Centerpoints (X-axis) to be Zero on Input.",
                                        style=wx.ALIGN_RIGHT,
                                        value=IMEX_CONFIG.get("FLAG_ZeroCenter", False)))
        manager.addrow(None)

        manager.addrow(WXtools.PrepCtrl(wx.Button, name="BTN_ExportFigure",
                                        label="Export Figure",
                                        tooltip="Export Figure as OBJ File\nto be morphed in external modeller.",
                                        font=labelfont,
                                        style=wx.BORDER_NONE,
                                        bgcolor=BTN_COLOR, fgcolor=(220, 200, 90),
                                        bind=(wx.EVT_LEFT_UP, self.onExportModel)),
                       WXtools.PrepCtrl(wx.Button, name="BTN_ImportFigure",
                                        label="Import Morph",
                                        tooltip="Import OBJ File as Morph",
                                        font=labelfont,
                                        style=wx.BORDER_NONE,
                                        bgcolor=BTN_COLOR, fgcolor=(100, 220, 190),
                                        bind=(wx.EVT_LEFT_UP, self.onImportModel)),
                       )

        manager.sizer.AddGrowableCol(1)
        sz = wx.BoxSizer(wx.VERTICAL)
        sz.Add(manager.sizer, 1, wx.EXPAND | wx.ALIGN_CENTER | wx.ALL, 10)
        self.SetSizerAndFit(sz)

    def updateFigurelist(self, ev=None):
        with wx.BusyInfo("Updating Figurelist"):
            while self.figurelist.GetCount():
                self.figurelist.Delete(0)

            for idx, fig in enumerate(poser.Scene().Figures()):
                self.figurelist.Append(fig.Name())
                if fig == poser.Scene().CurrentFigure():
                    self.figurelist.SetSelection(idx)
        self._CONFIGCHANGED = True

    def getScale(self):
        scale = wx.FindWindowByName("TX_Scale").GetValue()  # type: str
        return 1 if not scale.isnumeric() else int(scale)

    def onChoice(self, ev=None):
        pass

    def onClose(self, ev=None):
        try:
            IMEX_CONFIG["Perspective"] = aui.SavePaneInfo(paneInfo)
        except Exception as err:
            pass
        # force final saving
        self._CONFIGCHANGED = False
        IMEX_CONFIG.update(self.gb_manager.items_as_dict())
        save_config()

    def onExportModel(self, ev=None):
        figurename = self.figurelist.GetStringSelection()
        if figurename:
            aborted = False
            silent = wx.FindWindowByName("FLAG_SilentImExport", self).IsChecked()

            config_collection = IMEX_CONFIG.get(figurename, dict())  # filenames and parameters for one figure
            out_fname = config_collection.get("out_fname", "")
            if not silent or not out_fname or not os.path.isdir(config_collection.get("last_path", "")):
                with wx.FileDialog(self, message="Select file for Export",
                                   defaultDir=config_collection.get("last_path", ""),
                                   defaultFile=out_fname,
                                   wildcard="OBJ files (*.obj)|*.obj",
                                   style=wx.FD_DEFAULT_STYLE | wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT
                                   ) as dlg:
                    if dlg.ShowModal() == wx.ID_CANCEL:
                        aborted = True
                    else:
                        out_fname = dlg.GetPath()
                        config_collection["last_path"] = os.path.dirname(out_fname)
                        config_collection["out_fname"] = out_fname

            if not aborted:
                with wx.BusyInfo("Exporting '%s' to %s" % (figurename, config_collection.get("out_fname"))):
                    figure = SCENE.Figure(figurename)
                    IMEX_CONFIG["CTRL_MoveTextures"] = wx.FindWindowByName("FLAG_ImgCopy").IsChecked()
                    groups = IMEX_CONFIG["FLAG_UseGroups"] = wx.FindWindowByName("FLAG_UseGroups").IsChecked()
                    mats = IMEX_CONFIG["FLAG_UseMat"] = wx.FindWindowByName("FLAG_UseMat").IsChecked()
                    res = do_export(figure,
                                    filename=out_fname,
                                    scale=self.getScale(),
                                    use_groups=groups,
                                    use_mat=mats)

            IMEX_CONFIG[figurename] = config_collection
        else:
            WXtools.PopUpMessage("Figure '%s' not in Scene." % figurename,
                                 fg=LIGHT, bg=(0xf0, 00, 00))

        self._CONFIGCHANGED = True
        ev.Skip()

    def onImportModel(self, ev=None):
        figurename = self.figurelist.GetStringSelection()
        config_collection = IMEX_CONFIG.get(figurename, dict())
        try:
            SCENE.SelectFigure(SCENE.Figure(figurename))
        except poser.error:
            self.updateFigurelist(None)
            return WXtools.PopUpMessage("Figure '%s' not in Scene." % figurename,
                                        fg=LIGHT, bg=(0xf0, 00, 00))

        aborted = False
        silent = wx.FindWindowByName("FLAG_SilentImExport", self).IsChecked()

        in_fname = config_collection.get("in_fname", "")
        if not silent or not in_fname or not os.path.isdir(config_collection.get("last_path", "")):
            with wx.FileDialog(self, message="Select file for Import",
                               defaultDir=config_collection.get("last_path", THISPATH),
                               defaultFile=in_fname,
                               wildcard="OBJ files (*.obj)|*.obj",
                               style=wx.FD_DEFAULT_STYLE | wx.FD_OPEN
                               ) as dlg:
                if dlg.ShowModal() == wx.ID_CANCEL:
                    aborted = True
                else:
                    in_fname = dlg.GetPath()
                    config_collection["last_path"] = os.path.dirname(in_fname)
                    config_collection["in_fname"] = in_fname

        if not aborted:
            res = None
            if os.path.isfile(in_fname):
                with wx.BusyInfo("Importing Morph to Figure '%s'" % figurename):
                    try:
                        res = do_import(SCENE.CurrentFigure(),
                                        new_filename=in_fname,
                                        old_filename=config_collection.get("out_fname"),
                                        morphname=wx.FindWindowByName("TX_Morphname").GetValue(),
                                        force_center=wx.FindWindowByName("FLAG_ZeroCenter", self).IsChecked(),
                                        scale=self.getScale(),
                                        )
                    except Exception as err:
                        print(err)
                        res = None

            IMEX_CONFIG[figurename] = config_collection
            if res is None:
                WXtools.PopUpMessage("OBJ file not found or wrong file used.", closeafter=5)

        self._CONFIGCHANGED = True
        ev.Skip()


__timer__ = wx.Timer()


def tt():
    global __timer__
    try:
        if MainPanel._CONFIGCHANGED:
            MainPanel._CONFIGCHANGED = False
        save_config()
        __timer__ = wx.CallLater(1000, tt)

    except Exception:
        pass


if __name__ == "__main__":
    import wx.aui

    aui = poser.WxAuiManager()
    root = aui.GetManagedWindow()

    if not load_config():
        IMEX_CONFIG.update(dict(
            MORPH_ADD_TO_OLD_VALUE=True,
            Scale=1,
        ))

    paneInfo = aui.GetPane(TITLE)  # type: wx.aui.AuiPaneInfo
    if paneInfo.IsOk():
        aui.ClosePane(paneInfo)
        paneInfo = aui.GetPane(TITLE)

    savedinfo = IMEX_CONFIG.get("Perspective", None)
    if savedinfo is not None:
        aui.LoadPaneInfo(savedinfo, paneInfo)
    else:
        paneInfo.name = TITLE
        paneInfo.Movable(True).DestroyOnClose(True)

    paneInfo.Name(TITLE).Caption(TITLE)

    MainPanel = Menu(parent=root, name=TITLE)
    aui.AddPane(MainPanel, paneInfo)
    paneInfo.Show()
    aui.Update()
    tt()
